unit grapheNum;

interface

uses windows, sysUtils, graphics, classes, printers,
     Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs, math;

type
        pseudoWord = record case byte of
             0 : (octets : array[0..3] of ansiChar);
             1 : (mot : word);
             2 : (entier : smallInt);
             3 : (longMot : longWord);
             4 : (octet : byte);
        end;

const
        IntSansSignif = -1;
        MaxVecteur = 8192;
        pasTrace = 4;
        pasNiveau = 10;
        maxSon16 = 32764;
        Loge = 0.434294481903251828;
        CouleurFond = clWhite;
        MaxVecteurSon = 2097152; { 2 Mo }
        sansSignif : double = 1e-27;
        caracPrefixe : TSysCharSet =
    ['y','z','a','f','p','n','','m','k','M','G','T','P','E'];
        crSupprArr = char(vk_back);
        crTab = char(vk_tab);
        crCR = #13;
        crHaut = char(vk_up);
        crBas = char(vk_down);
        crGauche = char(vk_left);
        crDroite = char(vk_right);
        crHome = char(vk_home);
        crEnd = char(vk_end);

type
  CoefUnite = (zero,yocto,zepto,atto,femto,pico,nano,micro,milli,
               nulle,kilo,mega,giga,tera,peta,exa,zetta,yotta,infini);
        TformatWave = (f8,f16,f16stereo,f8stereo,f24,f24stereo);
        Tstereo = (gauche,droite);
        TvecteurSon = record
        case formatWave : TformatWave of
             f8 : (son8 : array[0..MaxVecteurSon] of byte);
             f16 : (son16 : array[0..MaxVecteurSon div 2] of smallInt);
             f16stereo : (son16s : array[0..MaxVecteurSon div 4,gauche..droite] of smallInt);
             f8stereo : (son8s : array[0..MaxVecteurSon div 2,gauche..droite] of byte);
        end;
        PvecteurSon = ^TvecteurSon;

 Tmonde = class
   mini,maxi : double;
   a,b : double; { entier = rel*a+b }
   constructor create;
   Procedure SetMinMax(min,max : double);
 end;

   TgrapheSon = class
                 Canvas : Tcanvas;
                 rectPB : Trect;
                 mondeX : Tmonde;
                 mondeY : Tmonde;
                 debut,fin,NbreTot : longInt;
                 LimiteCourbe : Trect;
                 Son,SonBis : PvecteurSon;
                 repere,repereTemps : integer;
                 pasFreq,NbreBits,NbreBitsInit : integer;
                 coeffBit : integer;
                 decimal,binaire : TlabeledEdit;
                 frequence : integer;
                 labelTemps : Tlabel;
                 decalageCanvas : integer;
                 avecTemps : boolean;
        Constructor Create;
        Procedure DrawG;
        Procedure DrawAll;
        Procedure Efface;
        Procedure DrawPoint(N : integer);
        Procedure WindowXY(xr,yr : double;var xi,yi : integer);
        Procedure MondeXY(xi,yi : integer;var xr,yr : extended);
        Procedure ChercheMonde;
        Procedure TraceRepere;
        function valeur24bits(j : longint) : integer;
        function valeur24(j : longint) : double;
        Function Xecran(xr : double) : integer;
        destructor Destroy;override;
   end;

Procedure VerifMinMaxInt(var mini,maxi : integer);
Procedure VerifMinMaxReal(var mini,maxi : double);
Function FloatToStrWin(valeur : double) : string;
Function StrToFloatWin(nombre : string) : double;
Function FloatToStrReg(valeur : double;NomUnite : string) : string;
Procedure FloatToValeurUnite(nombre : double;var valeurStr : string;
                                             var indexUnite : integer);
function formatBinary(nombre,precision : integer) : string;
function formatHexa(nombre,precision : integer) : string;
Function ValeurUniteToFloat(nombre : string;indexUnite : integer) : double;
Function StrToFloatReg(nombre : string;NomUnite : string) : double;


var
        CouleurCourbeBis : Tcolor = clRed;
        CouleurNiveau : Tcolor = clSilver;
        CouleurCourbe : Tcolor = clGray;

IMPLEMENTATION

const
prefixeUnite : array[coefUnite] of char =
       ('0','y','z','a','f','p','n','','m',
                ' ','k','M','G','T','P','E','Z','Y','');
valUnite : array[coefUnite] of double =
       (1E-27,1E-24,1E-21,1E-18,1E-15,1E-12,1E-9,1E-6,1E-3,
                1,1E+3,1E+6,1E+9,1E+12,1E+15,1E+18,1E+21,1E+24,1E+27);
expUnite : array[coefUnite] of string =
       ('E-27','E-24','E-21','E-18','E-15','E-12','E-9','E-6','E-3',
                '','E+3','E+6','E+9','E+12','E+15','E+18','E+21','E+24','E+27');
puissanceUnite : array[coefUnite] of shortInt =
       (-27,-24,-21,-18,-15,-12,-9,-6,-3,
                0,+3,+6,+9,+12,+15,+18,+21,+24,+27);

constructor Tmonde.Create;
begin
    inherited create;
    A := 1;
  	B := 0;
   	Maxi := 1;
   	Mini := 0;
end;

Procedure Tmonde.SetMinMax(min,max : double);
begin
	Maxi := max;
	Mini := min;
end;

Procedure TgrapheSon.MondeXY(xi,yi : integer;var xr,yr : extended);
begin
    	with mondeX do xr := debut+(xi-B)/A;
    	with mondeY do yr := (yi-B)/A
end;

Procedure TgrapheSon.WindowXY(xr,yr : double;var xi,yi : integer);
begin
      with mondeX do xi := round(A*(xr-debut)+B);
      with mondeY do yi := round(A*yr+B);
end;

procedure TgrapheSon.DrawG;

procedure TraceCourbe;
var x,y : integer;
    i : integer;
begin with LimiteCourbe,canvas do begin
      pen.Width := 1;
      pen.color := couleurCourbe;
      case son.formatWave of
         f8 : windowXY(debut,son.son8[debut],x,y);
         f16 : windowXY(debut,son.son16[debut],x,y);
         f8stereo : windowXY(debut,son.son8s[debut,gauche],x,y);
         f16stereo : windowXY(debut,son.son16s[debut,gauche],x,y);
      end;
      moveTo(x,y);
      case son.formatWave of
         f8 : for i := debut to fin do begin
            windowXY(i,son.son8[i],x,y);
            lineTo(x,y);
         end;
         f16 : for i := debut to fin do begin
            windowXY(i,son.son16[i],x,y);
            lineTo(x,y);
         end;
         f8stereo : for i := debut to fin do begin
            windowXY(i,son.son8s[i,gauche],x,y);
            lineTo(x,y);
         end;
         f16stereo : for i := debut to fin do begin
            windowXY(i,son.son16s[i,gauche],x,y);
            lineTo(x,y);
         end;
      end;
end end;// TraceCourbe

procedure TraceCourbeBis;
var x,y : integer;
    i : integer;
begin with LimiteCourbe,canvas do begin
      pen.width := 3;
      pen.color := couleurCourbeBis;
      case sonBis.formatWave of
         f8 : for i := debut to fin do begin
            windowXY(i,sonBis.son8[i],x,y);
            MoveTo(x,y);
            LineTo(x+pasTrace-1,y);
         end;
         f16 : for i := debut to fin do begin
             windowXY(i,sonBis.son16[i],x,y);
             MoveTo(x,y);
             LineTo(x+pasTrace-1,y);
         end;
         f8stereo : for i := debut to fin do begin
              windowXY(i,sonBis.son8s[i,gauche],x,y);
              MoveTo(x,y);
              LineTo(x+pasTrace-1,y);
         end;
         f16stereo : for i := debut to fin do begin
              windowXY(i,sonBis.son16s[i,gauche],x,y);
              MoveTo(x,y);
              LineTo(x+pasTrace-1,y);
         end;
      end;
end end; // TraceCourbeBis

procedure TraceNiveauFreq;
var x,x2,y : integer;
    i : integer;
begin with LimiteCourbe do begin
      windowXY(debut,0,x,y);
      windowXY(debut+pasFreq,0,x2,y);
      if (x2-x)<pasNiveau then exit;
      for i := debut to fin do if ((i mod pasFreq)=0) then begin
          windowXY(i,0,x,y);
          Canvas.MoveTo(x,top);
          Canvas.LineTo(x,bottom);
      end;
end end;// TraceNiveauFreq

procedure TraceNiveauBit;
var x,y,y2,s : integer;
begin with LimiteCourbe do begin
      coeffBit := round(power(2,NbreBitsInit-nbreBits));
      windowXY(debut,coeffBit,x,y);
      windowXY(debut,2*coeffBit,x,y2);
      if abs(y2-y)<pasNiveau then exit;
      if son.formatWave=f8
         then s := 0
         else s := -maxSon16;
      repeat
          windowXY(debut,s,x,y);
          Canvas.MoveTo(left,y);
          Canvas.LineTo(right,y);
          inc(s,coeffBit);
      until s>mondeY.maxi;
end end; // TraceNiveauBit

begin with canvas do begin
     brush.style := bsSolid;
     pen.style := psSolid;
     brush.color := couleurFond;
     FillRect(LimiteCourbe);
     pen.mode := pmCopy;
     if nbreTot<=0 then exit;
     ChercheMonde;
     pen.width := 1;
     pen.color := couleurNiveau;
     traceNiveauFreq;
     traceNiveauBit;
     traceCourbe;
     traceCourbeBis;
     traceRepere;
end end; // TgrapheSon.DrawG

procedure TgrapheSon.Efface;
begin with canvas do begin
     brush.style := bsSolid;
     pen.style := psSolid;
     brush.color := couleurFond;
     FillRect(LimiteCourbe);
end end; { TgrapheSon.Efface }

function TgrapheSon.valeur24bits(j : longint) : integer;
begin
  result := (Son.son8[j]+
             256*(Son.son8[j+1]+
                  256*Son.son8[j+2])) div 256;
end;

function TgrapheSon.valeur24(j : longint) : double; // entre -1 et +1
begin
  result := (Son.son8[j]+
             256*(Son.son8[j+1]+
                  256*Son.son8[j+2]))/8388608;
end;

procedure TgrapheSon.DrawPoint(N : integer);

procedure TraceCourbe;
var x1,y1,x2,y2 : integer;
begin with LimiteCourbe,canvas do begin
      pen.Width := 1;
      pen.color := couleurCourbe;
      case son.formatWave of
         f8 : windowXY(N,son.son8[N],x1,y1);
         f16 : windowXY(N,son.son16[N],x1,y1);
         f8stereo : windowXY(N,son.son8s[N,gauche],x1,y1);
         f16stereo : windowXY(N,son.son16s[N,gauche],x1,y1);
      end;
      moveTo(x1,y1);
      case son.formatWave of
         f8 : windowXY(N+1,son.son8[N+1],x2,y2);
         f16 : windowXY(N+1,son.son16[N+1],x2,y2);
         f8stereo : windowXY(N+1,son.son8s[N+1,gauche],x2,y2);
         f16stereo : windowXY(N+1,son.son16s[N+1,gauche],x2,y2);
      end;
      lineTo(x2,y2);
      if (N mod pasFreq)=0 then begin
          pen.color := couleurNiveau;
          MoveTo(x1,LimiteCourbe.bottom);
          LineTo(x1,y1);
      end;
end end; // TraceCourbe

procedure TraceCourbeBis;
var x,y : integer;
begin with LimiteCourbe,canvas do begin
      pen.width := 3;
      pen.color := couleurCourbeBis;
      case sonBis.formatWave of
         f8 : windowXY(N,sonBis.son8[N],x,y);
         f16 : windowXY(N,sonBis.son16[N],x,y);
         f8stereo : windowXY(N,sonBis.son8s[N,gauche],x,y);
         f16stereo : windowXY(N,sonBis.son16s[N,gauche],x,y);
      end;
      MoveTo(x,y);
      LineTo(x+pasTrace-1,y);
end end;{ TraceCourbeBis }

begin with canvas do begin
     brush.style := bsSolid;
     pen.style := psSolid;
     brush.color := couleurFond;
     pen.mode := pmCopy;
     ChercheMonde;
     pen.width := 1;
     pen.color := couleurNiveau;
     N := debut+N;
     traceCourbe;
     traceCourbeBis;
end end; // TgrapheSon.DrawPoint

Function TgrapheSon.Xecran(xr : double) : integer;
begin
    with mondeX do result := round(A*xr+B)
end;

procedure TgrapheSon.TraceRepere;
var x,y,xt : integer;
    valeur : integer;
    dt : double;{ en ms }
begin
if (NbreTot<=0) then exit;
with Canvas do begin
      Pen.Color := clRed;
      Pen.mode := pmXOR;
      Pen.width := 3;
      windowXY(repere,0,x,y);
      if (x>RectPB.left) and (x<RectPB.Right) then begin
         MoveTo(x,0);
         LineTo(x,RectPB.bottom);
         case son.formatWave of
              f8 : valeur := sonBis.son8[repere];
              f16 : valeur := sonBis.son16[repere];
              f8stereo : valeur := sonBis.son8s[repere,gauche];
              f16stereo : valeur := sonBis.son16s[repere,gauche];
              else valeur := 0; // pour le compilateur !
         end;
    // valeur de dpart
         if son.formatWave in [f16,f16stereo] then valeur := valeur+32768;
   // 16 bits avec signe, 8 bits sans
         valeur := valeur div coeffBit;
   // valeur quivalente en N bits
         Decimal.text := intToStr(valeur);
         Binaire.text := formatBinary(valeur,NbreBits);
         Binaire.hint := formatHexa(valeur,NbreBits);
      end
      else begin
         Decimal.text := '';
         Binaire.text := '';
         Binaire.hint := '';
         LabelTemps.visible := false;
         exit;
      end;
      Pen.Color := clYellow;
      Pen.width := 1;
      windowXY(repereTemps,0,xt,y);
      if avecTemps and
        (xt>RectPB.left) and
        (xt<RectPB.Right) then begin
         MoveTo(xt,0);
         LineTo(xt,RectPB.bottom);
         y := 20;
         MoveTo(x,y);
         LineTo(xt,y);
         dt := 1000*abs(repereTemps-repere)/frequence;
         if x<xt
            then LabelTemps.left := x + 8
            else LabelTemps.left := xt + 8;
         LabelTemps.visible := true;
         LabelTemps.top := y+DecalageCanvas-8;
         LabelTemps.caption := floatToStrF(dt,ffFixed,3,2)+' ms=1/('+intToStr(round(1000/dt))+' Hz)';
      end
      else LabelTemps.visible := false;
end end;

procedure TgrapheSon.ChercheMonde;
var Nbre : integer;
begin
    with mondeX,LimiteCourbe do begin
         A := pasTrace;
         Nbre := (right-left) div pasTrace;
         B := left;
     end;
     fin := debut+pred(Nbre);
     if fin>NbreTot then fin := NbreTot;
     with mondeY do
     if maxi=mini then if son.formatWave in [f8,f8stereo]
          then begin
              maxi := maxi + 16;
              if maxi>256 then maxi := 256;
              mini := mini - 16;
              if mini<0 then mini := 0;
          end
          else begin
              maxi := maxi + 256;
              if maxi>maxSon16 then maxi := maxSon16;
              mini := mini - 256;
              if mini<-maxSon16 then mini := -maxSon16
          end;
     if son.formatWave in [f8,f8stereo]
     then with LimiteCourbe,mondeY do begin
         A := (top-bottom)/(maxi-mini)/1.01;
         B := bottom;
     end
     else with LimiteCourbe,mondeY do begin
         A := (top-bottom)/(maxi-mini)/1.02;
         B := (top+bottom)/2;
     end
end;// ChercheMonde

destructor TgrapheSon.destroy;
begin
    mondeX.free;
    mondeY.free;
end;

procedure TgrapheSon.DrawAll;
var finS : integer;
    enveloppe : boolean;

procedure ChercheMondeAll;
begin
    with mondeX,LimiteCourbe do begin
         A := (right-left)/(finS);
         Enveloppe := A<0.2;
         B := left;
     end;
     case son.formatWave of
         f8,f8stereo : with LimiteCourbe,mondeY do begin
            A := (top-bottom)/256;
            B := bottom;
         end;
         f16,f16stereo : with LimiteCourbe,mondeY do begin
            A := (top-bottom)/65536;
            B := bottom+32768*A;
         end;
         f24,f24stereo : with LimiteCourbe,mondeY do begin
            A := (top-bottom)/65536;
            B := bottom+32768*A;
         end;
    end;
end;//  ChercheMonde

procedure TraceCourbe;
var x,y : integer;
    i,valeur : integer;
    pasS : integer;
begin with LimiteCourbe,mondeY do begin
      case son.formatWave of
         f8 : windowXY(0,son.son8[0],x,y);
         f16 : windowXY(0,son.son16[0],x,y);
         f24 : windowXY(0,valeur24bits(0),x,y);
         f8stereo : windowXY(0,son.son8s[0,gauche],x,y);
         f16stereo : windowXY(0,son.son16s[0,gauche],x,y);
         f24stereo : windowXY(0,valeur24bits(0),x,y);
      end;
      Canvas.moveTo(x,y);
      pasS := succ((finS) div MaxVecteur);
      case son.formatWave of
         f8 : for i := 0 to finS do if ((i mod pasS)=0) then begin
            valeur := son.son8[i];
            windowXY(i,valeur,x,y);
            Canvas.LineTo(x,y);
         end;
         f16 : for i := 0 to finS do if ((i mod pasS)=0) then begin
            valeur := son.son16[i];
            windowXY(i,valeur,x,y);
            Canvas.LineTo(x,y);
         end;
         f24 : for i := 0 to finS do if ((i mod pasS)=0) then begin
            valeur := valeur24bits(3*i);
            windowXY(i,valeur,x,y);
            Canvas.LineTo(x,y);
         end;
         f8stereo : for i := 0 to finS do if ((i mod pasS)=0) then begin
            valeur := son.son8s[i,gauche];
            windowXY(i,valeur,x,y);
            Canvas.LineTo(x,y);
         end;
         f16stereo : for i := 0 to finS do if ((i mod pasS)=0) then begin
            valeur := son.son16s[i,gauche];
            windowXY(i,valeur,x,y);
            Canvas.LineTo(x,y);
         end;
         f24stereo : for i := 0 to finS do if ((i mod pasS)=0) then begin
            valeur := valeur24bits(6*i);
            windowXY(i,valeur,x,y);
            Canvas.LineTo(x,y);
         end;
      end;
end end; // TraceCourbe

procedure TraceEnveloppe;
var x1,y1,x2,y2 : integer;
    i,j,pas,jdebut : integer;
    minSon,maxSon : integer;
    valeur : integer;
begin with LimiteCourbe,mondeY do begin
      pas := round(1/mondeX.A);
      for i := left to pred(right) do begin
          jdebut := i*pas;
          case son.formatWave of
             f8 : minSon := son.son8[jdebut];
             f16 : minSon := son.son16[jdebut];
             f24 : minSon := valeur24bits(3*jdebut);
             f8stereo : minSon := son.son8s[jdebut,gauche];
             f16stereo : minSon := son.son16s[jdebut,gauche];
             f24stereo : minSon := valeur24bits(6*jdebut);
             else minSon := 0; // pour le compilateur !
          end;{case}
          maxSon := minSon;
          for j := succ(jdebut) to pred(jdebut+pas) do
      case son.formatWave of
         f8 : if son.son8[j]>maxSon
                  then maxSon := son.son8[j]
                  else if son.son8[j]<minSon
                       then minSon := son.son8[j];
         f16 : if son.son16[j]>maxSon
                  then maxSon := son.son16[j]
                  else if son.son16[j]<minSon
                       then minSon := son.son16[j];
         f24 : begin
              valeur := valeur24bits(3*j);
              if valeur>maxSon
                  then maxSon := valeur
                  else if valeur<minSon
                       then minSon := valeur;
         end;
         f8stereo : if son.son8s[j,gauche]>maxSon
                  then maxSon := son.son8s[j,gauche]
                  else if son.son8s[j,gauche]<minSon
                       then minSon := son.son8s[j,gauche];
         f16stereo : if son.son16s[j,gauche]>maxSon
                  then maxSon := son.son16s[j,gauche]
                  else if son.son16s[j,gauche]<minSon
                       then minSon := son.son16s[j,gauche];
         f24stereo : begin
              valeur := valeur24bits(6*j);
              if valeur>maxSon
                  then maxSon := valeur
                  else if valeur<minSon
                       then minSon := valeur;
         end;
      end; // case format
      windowXY(jdebut,minSon,x1,y1);
      windowXY(jdebut,maxSon,x2,y2);
      Canvas.moveTo(i,y1);
      Canvas.lineTo(i,y2);
      end; // j
end end; // TraceEnveloppe

begin with canvas do begin
     brush.style := bsSolid;
     brush.color := couleurFond;
     pen.mode := pmCopy;
     pen.width := 1;
     pen.color := couleurCourbeBis;
     FillRect(LimiteCourbe);
     finS := pred(NbreTot);
     ChercheMondeAll;
     if enveloppe then traceEnveloppe else traceCourbe;
end end; // TgrapheSon.DrawAll

constructor TgrapheSon.Create;
begin
   inherited create;
   mondeX := Tmonde.create;
   mondeY := Tmonde.create;
   NbreTot := 0;
end;

Procedure VerifMinMaxInt(var mini,maxi : integer);
var z : integer;
begin
     if Maxi<Mini then begin
     	  z := Maxi;
    	  Maxi := Mini;
    	  Mini := z;
     end
end;

Procedure VerifMinMaxReal(var mini,maxi : double);
var z : double;
begin
     if Maxi<Mini then begin
	  z := Maxi;
	  Maxi := Mini;
	  Mini := z;
     end
end;

Function PuissEnt(X:double;N:Integer): double;

  Function PuissDeux(N:integer):double; { N>=0 }
  var Y : double;
  begin
     if N=0
        then PuissDeux := 1
        else begin
           Y := PuissDeux(N div 2);
	   Y := Sqr(Y);
 	   if (N mod 2)=0
	      then PuissDeux := Y
	      else PuissDeux := X*Y;
       end;
  end;{ puissDeux }

begin {PuissEnt}
    If N<0
       then PuissEnt := 1/PuissDeux(-N)
       else PuissEnt := PuissDeux(N)
end;{PuissEnt}

function Dix(x : integer) : double;
Begin
    Dix := PuissEnt(10,X)
end;

Function Log(X:double):double;
Begin
   Log := Ln(X)*loge
End;

Function StrToFloatWin(nombre : string) : double;

Procedure ConvertitPrefixe;
var u : coefUnite;
    c,c1 : char;
    posB : integer;
begin
   if length(nombre)<2 then exit;
   c := nombre[length(nombre)];
   c1 := nombre[pred(length(nombre))];
   if (c='m') and
      (charinset(c1,caracPrefixe) or (c1=' ')) then begin
        c := c1;
        delete(nombre,length(nombre),1); { mtre }
   end;
   if c='s' then begin
       c := c1;
       delete(nombre,length(nombre),1); { seconde }
   end;
   posB := pos(' ',nombre);
   while posb>0 do begin
         delete(nombre,posB,1);
         posB := pos(' ',nombre);
   end;
   if charinset(c,caracPrefixe) then begin
        u := zero;
        repeat inc(u) until (c=prefixeUnite[u]);
        delete(nombre,length(nombre),1);
        nombre := nombre+expUnite[u];
   end;
end;

var posSeparator : integer;
begin
   if formatSettings.decimalSeparator='.'
      then posSeparator := pos(',',nombre)
      else posSeparator := pos('.',nombre);
   if posSeparator>0 then Nombre[posSeparator] := formatSettings.DecimalSeparator;
   convertitPrefixe;
   result :=  strToFloat(nombre);
end; { StrToFloatWin }

Function ValeurUniteToFloat(nombre : string;indexUnite : integer) : double;
var posSeparator : integer;
begin
   if formatSettings.decimalSeparator='.'
      then posSeparator := pos(',',nombre)
      else posSeparator := pos('.',nombre);
   if posSeparator>0 then Nombre[posSeparator] := formatSettings.DecimalSeparator;
   try
   result :=  strToFloat(nombre);
   except
   result := 1;
   end;
   if indexUnite=1 then result := result/1000;
   if indexUnite=2 then result := result/1e6;
end;

Function StrToFloatReg(nombre : string;nomUnite : string) : double;
var long,position : integer;
begin
    long := length(nomUnite);
    position := length(nombre)-long+1;
    if (long>0) and
       (copy(nombre,position,long)=nomUnite)
          then delete(nombre,position,long);
    position := pos(' ',nombre);
    while position>0 do begin
        delete(nombre,position,1);
        position := pos(' ',nombre);
    end;
    try
    result := strToFloatWin(nombre);
    except
       result := sansSignif;
    end;
end;

Function FloatToStrWin(valeur : double) : string;
begin
    FloatToStrWin := FloatToStrF(valeur,ffGeneral,6,2)
end;

Function FloatToStrReg(valeur : double;nomUnite : string) : string;
const Precision = 5;
      MinReal = 1e-19;
var
   tampon  : string;
   unite   : coefUnite;
   PuissanceDix : double;
   Longueur : byte;
   AbsNombre : double;
   negatif : boolean;
begin
    negatif := valeur<0;
    absNombre := abs(valeur);
    if absNombre<minReal then begin
       result := '0';
       exit;
    end;
    try
    PuissanceDix := Dix(ceil(log(absNombre))-precision);
    except
       result := '0';
       exit;
    end;
    AbsNombre := round(AbsNombre/PuissanceDix)*PuissanceDix;
    unite := nulle;
    while ( AbsNombre>=999 ) and (unite<infini) do begin
          AbsNombre := AbsNombre/1000;
          inc(unite)
    end; { abs(nombre)<1000 ou infini }
    if unite=infini then begin
        result := '';
        exit
    end;
    while ( AbsNombre<0.999 ) and (unite>zero) do begin
          AbsNombre := AbsNombre*1000;
          Dec(unite)
    end; { abs(nombre)>=1 ou zero }
    if unite=zero then begin
        result := '0';
        exit
    end;
    longueur := succ(Precision);
 { succ pour le point }
    if AbsNombre<10
       then tampon := floatToStrF(AbsNombre,ffFixed,longueur,longueur-2)
 { 2 = un chiffre avant la virgule + virgule }
       else if AbsNombre<100
            then tampon := FloatToStrF(AbsNombre,ffFixed,longueur,longueur-3)
 { 3 =  deux chiffres avant la virgule + virgule }
            else tampon := FloatToStrF(AbsNombre,ffFixed,longueur,longueur-4);
 { 4 =  trois chiffres avant la virgule + virgule }
      if pos('NAN',tampon)<>0 then begin
           { Not a Number }
             result := '';
             exit;
      end;
      while tampon[1]=' ' do delete(tampon,1,1);
      longueur := pos('.',tampon);
      if longueur<=Precision
         then tampon := copy(tampon,1,Precision+1)
         else tampon := copy(tampon,1,Precision);
   tampon := tampon+' ';      
   if negatif then tampon := '-' + tampon;
   if unite<>nulle then tampon := tampon+prefixeUnite[unite];
   result := tampon+nomUnite;
end;{FloatToStrReg}

Procedure FloatToValeurUnite(nombre : double;var valeurStr : string;
                                             var indexUnite : integer);
const Precision = 5;
      MinReal = 1e-15;
var
   tampon  : string;
   unite   : coefUnite;
   PuissanceDix : double;
   Longueur : byte;
begin
    nombre := abs(nombre);
    valeurStr := '1';
    indexUnite := 0;
    if nombre<minReal then exit;
    try
    PuissanceDix := Dix(ceil(log(nombre))-precision);
    except
       exit;
    end;
    nombre := round(nombre/PuissanceDix)*PuissanceDix;
    unite := nulle;
    while ( nombre>=999 ) and (unite<infini) do begin
          nombre := nombre/1000;
          inc(unite)
    end; { abs(nombre)<1000 ou infini }
    if unite=infini then exit;
    while ( nombre<0.999 ) and (unite>zero) do begin
          nombre := nombre*1000;
          Dec(unite)
    end; { abs(nombre)>=1 ou zero }
    if unite=zero then exit;
    longueur := succ(Precision);
 { succ pour le point }
    if nombre<10
       then tampon := FloatToStrF(nombre,ffFixed,longueur,longueur-2)
 { 2 = un chiffre avant la virgule + virgule }
       else if nombre<100
            then tampon := FloatToStrF(nombre,ffFixed,longueur,longueur-3)
 { 3 =  deux chiffres avant la virgule + virgule }
            else tampon := FloatToStrF(nombre,ffFixed,longueur,longueur-4);
 { 4 =  trois chiffres avant la virgule + virgule }
      if pos('NAN',tampon)<>0 then begin
           { Not a Number }
             exit;
      end;
      while tampon[1]=' ' do delete(tampon,1,1);
      longueur := pos('.',tampon);
      if longueur<=Precision
         then tampon := copy(tampon,1,Precision+1)
         else tampon := copy(tampon,1,Precision);
   valeurStr := tampon;
   if unite<micro then unite := micro;
   if unite>nulle then unite := nulle;
   case unite of
        micro : indexUnite := 2;
        milli : indexUnite := 1;
        nulle : indexUnite := 0;
        else indexUnite := 0;
  end;
end; // FloatToValeurUnite

const
      Digits : array[0..$F] of Char = '0123456789ABCDEF';

function formatBinary(nombre,precision : integer) : string;
    {-Return binary string for word}
  var
    I,N,W : Word;
  begin
    if nombre>=0
       then W := nombre
       else W := nombre+32768; { complment  2 }
    N := 1;
    setLength(result,precision);
    for i := pred(Precision) downto 0 do begin
      result[N] := Digits[Ord(W and (1 shl I) <> 0)]; {0 or 1}
      Inc(N);
    end;
  end;

  function formatHexa(nombre,precision : integer) : string;
    {-Return hex string for word}
  var masque,i,j,W : word;
  begin
    W := round(nombre);
    Masque := 0;
    j := 1;
    for i := 1 to Precision do begin
        Masque := Masque+j;
        j := j*2;
    end;
    W := W and masque;
    result := Digits[lo(W) and $F]+'h';
    if Precision>4 then
       result := Digits[lo(W) shr 4]+result;
    if Precision>8 then
       result := Digits[hi(W) and $F]+result;
    if Precision>12 then
       result := Digits[hi(W) shr 4]+result;
  end;

initialization
  couleurNiveau := RGB(224,224,224)
end.

